#ifndef ANALISADOR_H
#define ANALISADOR_H

#include <cv.h>
#include <highgui.h>

/*
  A classe Analisador é a parte plugável do sistema: ela oferece os
  pontos de entrada para os algoritmos a serem usados na análise dos
  panfletos. A ideia é usar algo parecido com o design pattern Template Method.
  As funções de cada método do Analisador está descrito a seguir:

  segmenta_fundo: recebe uma imagem do panfleto sem nenhum pré-processamento e
    e modifica para deixar o fundo da imagem na cor preta (e tudo que não é fundo
    na cor original). A entrada pode ser colorida ou em níveis de cinza.

  encontra_produtos: recebe a imagem do passo anterior e encontra os produtos na imagem.
    Esta função deve retornar um CvContour com cada contorno externo dos produtos.

  recorte_rect: recebe um produto e define qual será o recorte feito neste produto para
      poder encontrar o nome dele e seu preço.

  encontra_nomes_de_produtos: recebe um recorte da imagem original centrada em um produto
    (o tamanho do recorte é configurável reescrevendo a função recorte_rect) e o CvContour de um produto
    e devolve uma máscara com os textos que são a descrição do produto.

  encontra_precos: recebe uma imagem que só possui o produto e o que não foi selecionado
    na função acima e um CvContour (que é o produto) e devolve uma máscara que selecione
    o preço do produto.

  Estão funções são chamadas pelo programa escrito no main e formam a parte customizável do
  sistema: trocando estas partes é possível analisar diversos tipos de panfletos. Além disto,
  uma parte do trabalho já é feita no main (como leitura das imagens usando o tesseract e
  recortes para analisar partes separadas da imagem) e o a ideia é que a única parte que
  deveria ser modificada para suportar novos panfletos é fazer uma classe que herda de
  Analisador e reescrever os métodos dela. Um exemplo de como isto pode ser feito é a classe
  AnalisadorPDA, que analisa panfletos do Pão de Açúcar.

  */

class Analisador {
public:
    Analisador();
    virtual IplImage *segmenta_fundo (IplImage *pagina);
    virtual CvContour *encontra_produtos (IplImage *pagina, IplImage *fundo);
    virtual CvRect recorte_rect(IplImage *pagina, CvContour *produto);
    virtual void encontra_nomes_de_produtos (IplImage *recorte, IplImage *recorte_fundo, CvContour *produto);
    virtual void encontra_precos (IplImage *recorte, CvContour *produto);
};

#endif // ANALISADOR_H
