#include "analisadorpda.h"

#include <cv.h>
#include <cstdlib>
#include "contrast.h"
#include "produto.h"
#include "definitions.h"

AnalisadorPDA::AnalisadorPDA() : Analisador ()
{
    this->contraste = NULL;
    /* cria os dois classificadores */
    this->class_nomes = treina_classificador("./treinamento_imagens/treinamento_ordenado.csv");
}

void AnalisadorPDA::detecta_linhas(IplImage *pagina)
{
    IplImage *cinza = cvCloneImage(pagina); /*cvCreateImage(cvGetSize(pagina), IPL_DEPTH_8U, 1);

    cvCvtColor(pagina, cinza, CV_RGB2GRAY);*/
    cvThreshold(cinza, cinza, 60, 255, cv::THRESH_BINARY);
    cvNot(cinza, cinza);
    this->linhas = cvCreateImage(cvGetSize(pagina), IPL_DEPTH_8U, 1);
    cvZero(this->linhas);

    /* parâmetros para o tamanho esperado de uma linha */
    int min_size = 390, line_thickness = 13;

    /* Linhas horizontais: */
    IplImage *hor = cvCloneImage(cinza);
    cvMorphologyEx(cinza, hor, NULL,
        cvCreateStructuringElementEx(1, line_thickness, 0, line_thickness/2, CV_SHAPE_RECT, NULL), CV_MOP_BLACKHAT, 1);
    cvDilate(hor, hor, NULL, 2);
    cvMorphologyEx(hor, hor, NULL,
        cvCreateStructuringElementEx(min_size, 1, min_size/2, 0, CV_SHAPE_RECT, NULL), CV_MOP_OPEN, 1);

    cvMorphologyEx(hor, hor, NULL,
                     cvCreateStructuringElementEx(500, 1, 250, 0, CV_SHAPE_RECT, NULL), CV_MOP_CLOSE, 1);

    /* Linhas verticais: */
    IplImage *vert = cvCloneImage(cinza);
    cvMorphologyEx(cinza, vert, NULL,
        cvCreateStructuringElementEx(line_thickness, 1, line_thickness/2, 0, CV_SHAPE_RECT, NULL), CV_MOP_BLACKHAT, 1);
    cvDilate(vert, vert, NULL, 2);
    cvMorphologyEx(vert, vert, NULL,
        cvCreateStructuringElementEx(1, 20, 0, 10, CV_SHAPE_RECT, NULL), CV_MOP_CLOSE, 1);
    cvMorphologyEx(vert, vert, NULL,
        cvCreateStructuringElementEx(1, min_size, 0, min_size/2, CV_SHAPE_RECT, NULL), CV_MOP_OPEN, 1);
    cvMorphologyEx(vert, vert, NULL,
                     cvCreateStructuringElementEx(1, 300, 0, 150, CV_SHAPE_RECT, NULL), CV_MOP_CLOSE, 1);

    cvOr(vert, hor, this->linhas);
    cvReleaseImage(&hor);
    cvReleaseImage(&vert);
    cvDilate(this->linhas, this->linhas, NULL, 10);
    cvSaveImage("linhas.png", this->linhas, 0);
}

IplImage *AnalisadorPDA::segmenta_fundo(IplImage *pagina) {
    cvSaveImage("original.png", pagina, 0);

    processa_usando_piramide(pagina, &(this->contraste), 2);
    cvSaveImage("contraste.png", this->contraste, 0);
    IplImage *fundo = cvCreateImage(cvGetSize(pagina), IPL_DEPTH_8U, 1);
    cvThreshold(this->contraste, fundo, 40, 255, cv::THRESH_BINARY);
    this->detecta_linhas(this->contraste);
    /* Tira ruido */
    cvMorphologyEx(fundo, fundo, NULL, NULL, cv::MORPH_OPEN, 1);
    /* Tira linhas */
    cvMorphologyEx(fundo, fundo, NULL,
                   cvCreateStructuringElementEx(17, 1, 8, 0, cv::MORPH_RECT, NULL), cv::MORPH_OPEN, 1);
    cvMorphologyEx(fundo, fundo, NULL,
                   cvCreateStructuringElementEx(1, 17, 0, 8, cv::MORPH_RECT, NULL), cv::MORPH_OPEN, 1);
    cvSaveImage("fundo.png", fundo, 0);
    return fundo;
}

CvContour *AnalisadorPDA::encontra_produtos(IplImage *pagina, IplImage *fundo)
{
    CvContour *cc = (CvContour *) connected_components(fundo);
    cc = pre_processa_cc(cc, 2.0, 2.0, BIGGER);
    return cc;
}

CvRect AnalisadorPDA::recorte_rect(IplImage *pagina, CvContour *produto)
{
    CvRect r;
    IplImage *mask = cvCreateImage(cvSize(pagina->width + 2, pagina->height + 2), IPL_DEPTH_8U, 1);
    cvZero(mask);
    CvConnectedComp cc;
    CvPoint seed; seed.x = produto->rect.x; seed.y = produto->rect.y;
    cvFloodFill(this->linhas, seed, cvScalar(255), cvScalar(0), cvScalar(0), &cc, 4 | CV_FLOODFILL_MASK_ONLY, mask);
    cvReleaseImage(&mask);
    return cc.rect;
}


void AnalisadorPDA::encontra_nomes_de_produtos(IplImage *recorte, IplImage *recorte_fundo,  CvContour *produto)
{
    char name[50];
    IplImage *contraste_recorte;
    IplImage *cinza = cvCreateImage(cvGetSize(recorte), IPL_DEPTH_8U, 1);
    cvCvtColor(recorte, cinza, CV_BGR2GRAY);
    IplImage *th = cvCreateImage(cvGetSize(recorte), IPL_DEPTH_8U, 1);
    CvRect regiao_cc;
    pre_processa_imagem_para_classificacao(cinza, th, PRODUTOS_BLOCK, PRODUTOS_DELTA);
    processa_usando_piramide(recorte, &contraste_recorte, 2);
    cvDrawContours(th, (CvSeq *) produto, CV_RGB(0, 0, 0), CV_RGB(0, 0, 0), -1, CV_FILLED, 8,
        cvPoint(0, 0));
    CvContour *cc = (CvContour *) connected_components(th);
    float caracteristicas[8];
    for (CvContour *t = cc; t != NULL; t = (CvContour *) t->h_next) {
        regiao_cc.x = t->rect.x;// + recorte->roi->xOffset;
        regiao_cc.y = t->rect.y;// + recorte->roi->yOffset;
        regiao_cc.width = t->rect.width;
        regiao_cc.height = t->rect.height;
        extrai_caracteristicas(contraste_recorte, th, &regiao_cc, caracteristicas);
        /* classifica como nome de produto ou qq outra coisa */
        if (classifica(this->class_nomes, caracteristicas, 8) == CLASS_TEXTO) {
            /*cvRectangle(recorte, cvPoint(t->rect.x, t->rect.y),
                cvPoint(t->rect.width + t->rect.x - 1, t->rect.height + t->rect.y - 1),
                CV_RGB(150, 0, 0), 3, 8, 0);*/
//            cvDrawContours(recorte, (CvSeq *) t, CV_RGB(150, 0, 0), CV_RGB(0, 0, 0), -1, CV_FILLED, 8,
//                cvPoint(0, 0));
        }
    }
    sprintf(name, "%d.png", rand());
    cvSaveImage(name, recorte, 0);
    cvReleaseImage(&cinza);
    cvReleaseImage(&contraste_recorte);
    cvReleaseImage(&th);
}

