#include "util.h"

#include <cv.h>
#include <highgui.h>
#include <math.h>
#include <stdio.h>

IplImage *contrast_map(IplImage *img, int theta, double sigma)
{
    int i, j, k, l, max = 0;
    int r, g, b, r2, g2, b2, *imageData32bit;
    IplImage *contrast32bit = cvCreateImage(cvGetSize(img), IPL_DEPTH_32S, 1);
    IplImage *contrast8bit = cvCreateImage(cvGetSize(img), IPL_DEPTH_8U, 1);
    cvZero(contrast32bit);
    for (i = theta; i < img->height - theta; i++) {
        for (j = theta; j < img->width - theta; j++) {
            r = pixel_at(img, i, j, 0);
            g = pixel_at(img, i, j, 1);
            b = pixel_at(img, i, j, 2);
            for (k = -theta; k < theta; k++) {
                for (l = -theta; l < theta; l++) {
                    r2 = pixel_at(img, i + k, j + l, 0);
                    g2 = pixel_at(img, i + k, j + l, 1);
                    b2 = pixel_at(img, i + k, j + l, 2);
                    imageData32bit = (int *) (&(contrast32bit->imageData[contrast32bit->widthStep * i]));
                    imageData32bit[j] += gaussian_distance(r, g,b, r2, g2, b2, SIGMA);
                }
            } 
            imageData32bit = (int *) (&(contrast32bit->imageData[contrast32bit->widthStep * i]));
            if (imageData32bit[i] > max)
                max = imageData32bit[j];
        }
    }
    cvConvertScale(contrast32bit, contrast8bit, 255.0 / max, 0);
    return contrast8bit;
}

IplImage *processa_nivel_abaixo(IplImage *img, IplImage **img_half, int window, int sigma)
{
    IplImage *half = cvCreateImage(cvSize(img->width/2, img->height/2), IPL_DEPTH_8U, img->nChannels);
    IplImage *contrast;
    cvPyrDown(img, half, CV_GAUSSIAN_5x5);
    contrast = contrast_map(half, window, sigma);
    *img_half = half;
    return contrast;
}

uchar calcula_nivel_acima(int valor_i, int valor_i_1, double peso)
{
    int r;
    int val = valor_i;
    val += peso * valor_i_1;
    r = (int) val/(peso+1);
    return r;
}

void junta_niveis(IplImage *nivel_i, IplImage *nivel_i1, double peso) 
{
    int i, j, val_i, val_i1;
    int i_i, i_j, i1_i, i1_j;
    for (i = 0; i < nivel_i1->height; i++) {
        for (j = 0; j < nivel_i1->width; j++) {
            val_i1 = pixel_at(nivel_i1, i, j, 0);
            
            val_i = pixel_at(nivel_i, 2 * i, 2 * j, 0);
            set_pixel_at(nivel_i, 2 * i,  2 * j, 0, calcula_nivel_acima(val_i, val_i1, peso));
            
            val_i = pixel_at(nivel_i, 2 * i + 1, 2 * j, 0);
            set_pixel_at(nivel_i, 2 * i + 1,  2 * j, 0, calcula_nivel_acima(val_i, val_i1, peso));
            
            val_i = pixel_at(nivel_i, 2 * i, 2 * j + 1, 0);
            set_pixel_at(nivel_i, 2 * i,  2 * j + 1, 0, calcula_nivel_acima(val_i, val_i1, peso));
            
            val_i = pixel_at(nivel_i, 2 * i + 1, 2 * j + 1, 0);
            set_pixel_at(nivel_i, 2 * i + 1,  2 * j + 1, 0, calcula_nivel_acima(val_i, val_i1, peso));
            
        }
    }   
}

CvContour *processa_usando_piramide(IplImage *img, IplImage **contrast, int w)
{
    CvContour *cc;
    IplImage *nivel2 = NULL;
    IplImage *nivel3 = NULL;
    IplImage *c1, *c1_clone, *c2, *c3;
    if (img->width < 4 * w || img->height < 4 * w) {
        *contrast = cvCloneImage(img);
        cvZero(*contrast);
        return NULL;
    }
    c1 = contrast_map(img, w, 10);
    c1_clone = cvCloneImage(c1);
    c2 = processa_nivel_abaixo(img, &nivel2, w, 10);
    c3 = processa_nivel_abaixo(nivel2, &nivel3, w, 10);
    junta_niveis(c2, c3, 6);
    junta_niveis(c1, c2, 2);
    *contrast = cvCloneImage(c1);
    cvThreshold(c1, c1, 100, 255, CV_THRESH_BINARY);
    
    cvMorphologyEx(c1, c1, NULL, 
        cvCreateStructuringElementEx(9, 9, 4, 4, CV_SHAPE_RECT, NULL), CV_MOP_OPEN, 1);
    cvMorphologyEx(c1, c1, NULL, 
        cvCreateStructuringElementEx(9, 9, 4, 4, CV_SHAPE_RECT, NULL), CV_MOP_CLOSE, 1);
        
    cc = pre_processa_cc(connected_components(c1), 0.3, 0.3, SMALLER);
    cvReleaseImage(&c1);
    cvReleaseImage(&c1_clone);
    cvReleaseImage(&c2);
    cvReleaseImage(&c3);
    return cc;
}

CvContour *processa_normal(IplImage *img)
{
    CvContour *cc;
    IplImage *c1 = contrast_map(img, 10, 10);
    cvThreshold(c1, c1, 100, 255, CV_THRESH_BINARY);
    cvMorphologyEx(c1, c1, NULL, 
        cvCreateStructuringElementEx(9, 9, 4, 4, CV_SHAPE_RECT, NULL), CV_MOP_OPEN, 1);
    cc = pre_processa_cc(connected_components(c1), 0.3, 0.3, SMALLER);
    return cc;
}


