#include <string.h>
#include <stdio.h>
#include <cv.h>
#include <highgui.h>
#include "util.h"
#include "definitions.h"
#include "classifier.hpp"

#define N_SAMPLES 8

#define N_CC 206

#define CAMINHO "./treinamento_imagens"

void gerar_imagens()
{
    char img_name[256];
    int i = 0, j = 0;
    IplImage *color = NULL, *gray, *mask = NULL, *mask2 = NULL, *contraste;
    CvContour *cc, *t;
    
    for (i = 0; i < N_SAMPLES; i++) {
        sprintf(img_name, CAMINHO "/imagens_usadas/c%d.png", i);
        printf("Abrindo %s\n", img_name);
        /*
           TODO: adicionar preenchimento da região ao treinamento
        */
        color = cvLoadImage(img_name, 1);
        processa_usando_piramide(color, &contraste, 2);
        gray = cvLoadImage(img_name, 0);
        mask = cvCloneImage(gray);
        pre_processa_imagem_para_classificacao(gray, mask, PRODUTOS_BLOCK, PRODUTOS_DELTA);
        cc = (CvContour *) connected_components(mask);
        for (t = cc; t != NULL; t = (CvContour *) t->h_next, j++) {
            cvBoundingRect(t, 1);
            if (t->rect.width <= 10 && t->rect.height <= 10) {
                if (rand() % 10 <= 8) {
                    j--;
                    continue;
                }
            }
            printf("Contorno %d: (%d, %d), sz (%d, %d)\n", j, t->rect.x, t->rect.y, t->rect.width, t->rect.height);
            
            cvSetImageROI(mask, t->rect);
            cvSetImageROI(contraste, t->rect);
            
            sprintf(img_name, CAMINHO "/componentes_conexas/c%d.png", j);
            cvSaveImage(img_name, mask, NULL);
            sprintf(img_name, CAMINHO "/componentes_conexas/c%d-contraste.png", j);
            cvSaveImage(img_name, contraste, NULL);
            cvSetImageROI(mask, cvRect(0, 0, mask->width, mask->height));
            cvSetImageROI(contraste, cvRect(0, 0, contraste->width, contraste->height));
            fflush(stdout);
        }
    }
}

void constroi_treinamento () 
{
    char img_name[256];
    int i = 0;
    float media;
    IplImage *img = NULL, *cc = NULL;
    CvRect all;
    FILE *f = fopen(CAMINHO "/treinamento.csv", "w");
    fprintf(f, "%d\n", N_CC);
    for (i = 0; i < N_CC; i++) {
        sprintf(img_name, CAMINHO "/componentes_conexas/c%d.png", i);
        printf("Abrindo %s\n", img_name);
        img = cvLoadImage(img_name, 0);
        sprintf(img_name, CAMINHO "/componentes_conexas/c%d-contraste.png", i);
        cc = cvLoadImage(img_name, 0);
        all = cvRect(0,0, img->width, img->height);
        /* captura caracteristicas e coloca em um arquivo.
         * ORDEM: CLASSE, VIZINHOS1, VIZINHOS2, PREENCHIMENTO, ASPECTO, 
         *   MEDIA_CONTRASTE, DP_CONTRASTE, LARGURA, ALTURA */
        /* se o tamanho é menor que 10x10 é ruído. Coloca classe automaticamente */
        if (img->width < 10 && img->height < 10) {
            fprintf(f, "%d;", CLASS_NTEXTO);
        } else {
            fprintf(f, "-----");
        }
        /* nao calculo vizinhos1, 2 e preenchimento. */
        media = media_regiao(&all, cc);
        fprintf(f, "0.0;0.0;0.0;%f;%f;%f;%f;%f\n", 
            1.0 * img->width / img->height, 
            media, 
            dp_regiao(&all, cc, media),
            1.0 * img->width, 
            1.0 * img->height
            );
        
        cvReleaseImage(&img);
    }    
    fclose(f);
}

void atualiza_treinamento() 
{
    char img_name[256];
    int i = 0, classe;
    float media, preenchimento, lixo;
    IplImage *img = NULL, *cc = NULL;
    CvRect all;
    FILE *f = fopen(CAMINHO "/treinamento.csv", "r");
    FILE *w = fopen(CAMINHO "/treinamento_atualizado.csv", "w");
    fprintf(w, "%d\n", N_CC);
    for (i = 0; i < N_CC; i++) {
        if (fscanf(f, "%d;%f;%f;%f;%f;%f;%f;%f;%f\n", &classe, &lixo, &lixo, &lixo, &lixo, &lixo, &lixo, &lixo, &lixo) == EOF) {
            classe = -1;
        }
        sprintf(img_name, CAMINHO "/componentes_conexas/c%d.png", i);
        printf("Abrindo %s\n", img_name);
        img = cvLoadImage(img_name, 0);
        sprintf(img_name, CAMINHO "/componentes_conexas/c%d-contraste.png", i);
        cc = cvLoadImage(img_name, 0);
        all = cvRect(0,0, img->width, img->height);
        /* captura caracteristicas e coloca em um arquivo.
         * ORDEM: CLASSE, VIZINHOS1, VIZINHOS2, PREENCHIMENTO, ASPECTO, 
         *   MEDIA_CONTRASTE, DP_CONTRASTE, LARGURA, ALTURA */
        /* se o tamanho é menor que 10x10 é ruído. Coloca classe automaticamente */
        fprintf(w, "%d;", classe);
        /* nao calculo vizinhos1, 2 e preenchimento. */
        media = media_regiao(&all, cc);
        fprintf(w, "0.0;0.0;%f;%f;%f;%f;%f;%f\n",
            preenchimento_regiao(&all, img),
            1.0 * img->width / img->height, 
            media, 
            dp_regiao(&all, cc, media),
            1.0 * img->width, 
            1.0 * img->height
            );
        cvReleaseImage(&img);
    }    
    fclose(f);
    fclose(w);
}

int main(int argc, char *argv[])
{
    if (strcmp(argv[1], "gerar") == 0) {
        printf("gerando imagens dos componentes conexos\n");
        gerar_imagens();
    } else if (strcmp(argv[1], "contruir") == 0) {
        printf("Construir arquivo de treinamento: necessita de classificação manual\n");
        constroi_treinamento();
    } else if (strcmp(argv[1], "atualizar") == 0) {
        printf("Atualizando treinamento: manter mesma classe mas atualizar atributos\n");
        atualiza_treinamento();
    }
    return 0;
}

